/*
 * Copyright 2016 - 2025 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <imgui.h>

#include "projectexplorer.hpp"
#include "projecttreemodel.hpp"

namespace studio {

ProjectTreeModel::ProjectTreeModel(
	ProjectExplorer &explorer,
	ox::StringParam name,
	ProjectTreeModel *parent) noexcept:
	m_explorer(explorer),
	m_parent(parent),
	m_name(std::move(name)) {
}

ProjectTreeModel::ProjectTreeModel(ProjectTreeModel &&other) noexcept:
	m_explorer(other.m_explorer),
	m_parent(other.m_parent),
	m_name(std::move(other.m_name)),
	m_children(std::move(other.m_children)) {
}

bool ProjectTreeModel::draw(turbine::Context &ctx) const noexcept {
	bool updated = false;
	constexpr auto dirFlags = ImGuiTreeNodeFlags_OpenOnArrow | ImGuiTreeNodeFlags_OpenOnDoubleClick;
	if (!m_children.empty()) {
		if (ImGui::TreeNodeEx(m_name.c_str(), dirFlags)) {
			for (auto const&child : m_children) {
				updated = child->draw(ctx) || updated;
			}
			ImGui::TreePop();
		}
	} else {
		auto const path = fullPath();
		auto const name = ox::sfmt<ox::BasicString<255>>("{}##{}", m_name, path);
		if (ImGui::TreeNodeEx(name.c_str(), ImGuiTreeNodeFlags_Leaf)) {
			if (ImGui::IsItemHovered() && ImGui::IsMouseDoubleClicked(0)) {
				m_explorer.fileChosen.emit(path);
			}
			if (ImGui::BeginPopupContextItem("FileMenu", ImGuiPopupFlags_MouseButtonRight)) {
				if (ImGui::MenuItem("Delete")) {
					updated = m_explorer.romFs()->remove(path).errCode == 0 || updated;
				}
				ImGui::EndPopup();
			}
			ImGui::TreePop();
		}
	}
	return updated;
}

void ProjectTreeModel::setChildren(ox::Vector<ox::UPtr<ProjectTreeModel>> children) noexcept {
	m_children = std::move(children);
}

ox::BasicString<255> ProjectTreeModel::fullPath() const noexcept {
	if (m_parent) {
		return m_parent->fullPath() + "/" + ox::StringView(m_name);
	}
	return {};
}

}
